unit TMWMsg;
(*
  The TMessagePanel component is used simply to display messages on a panel
  for any form. Just tell it how to act and voila!, a nice little message
  display panel. There are settings for normal message display and for error
  message display. You can set the font and color scheme of each one. You can
  also control the panels size and position manually if you like. It has options
  that allow it to center itself on it's parent control and also resize itself
  based on the displayed string. If you turn off the automatic position or sizing,
  then you must set the associated properties (height, width, top, left) manually.

  There are 2 ways to display information, using either the DisplayInfo
  procedure or the DisplayInfoPause procedure. The DisplayInfo procedure
  takes 2 parameters: s: string; Error: boolean. If the string passed is empty,
  the panel is hidden. If it isn't blank, then the message will be displayed
  using the font and color based on the Error parameter. Error = true will
  use the error font and color.

  Here are some examples to go by:

       ex: Normal message displays

          DisplayInfo('Querying database now...');
          .. do stuff here..
          DisplayInfo('');

       ex: Error message displays

          DisplayInfo('Error while querying data...', true);
          .. do stuff here ..
          DisplayInfo('');

       ex: display message for 2.5 seconds

          DisplayInfoPause('Please rethink your decision.', 2.5);

       ex: display error message for 5 seconds

          DisplayInfoPause('Error, Please leave me alone now.', 5, true);
*)
interface
  uses
    Windows, Forms, Graphics, SysUtils, Classes, Controls, ExtCtrls, StdCtrls;

type

  // MyPnl is used to access the canvas of the TPanel object for
  // determining the height and width of text displayed
  MyPnl = class(TPanel)
  public
  	property Canvas;
  end;

  TMessagePanel = class(TComponent)
  private
    fOwner: TComponent;
    DisplayPanel: TPanel;
    fMessageInfo: string;
    fErrorCode, fAutoCenter,
    fAutoSize, fKeepPause,
    fDotsOn: boolean;
    eColor, fColor: TColor;
    eFont, fFont: TFont;
    fMargin, fLeft, fTop, fHeight,
    fMaxPause, fMinWidth, fWidth: integer;
    fDotsInterval: double;
    fOnDestroy, fOnChange: TNotifyEvent;
    fDotLabel: TLabel;
    fTimer: TTimer;

  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;

    // DisplayInfo(s: string): used to display any string s, the optional
    //   Error parameter tells it to use standard (default) font and
    //   colors or error settings. To clear the message and hide the
    //   message panel, call DisplayInfo with a blank string
    procedure DisplayInfo(s: string; Error: boolean = false);
    // DisplayInfoPause(s: string; m: double): used to display a string s
    //   for a given m number of seconds, it will then reset the message
    //   to whatever it was before calling DisplayInfoPause. If there was
    //   no previous message being displayed, then it will hide the panel
    procedure DisplayInfoPause(s: string; m: double; Error: boolean = false);
    // GetDisplayInfo: string: used to retrieve the current string being
    //   displayed
    function GetDisplayInfo: string;

    property PanelParent: TComponent read fOwner write fOwner;
  private
    procedure DotTimer(Sender: TObject);
  published
    (* PROPERTIES
        AutoCenter: set to true to let component center itself
        AutoSize: set to true to let component size itself based on text
        Color: the panel color for normal messages
        DotsInterval: seconds to wait between blinking dots
        DotsOn: true/false depending on whether to show blinking dots
        ErrorColor: the color of the panel for error messages
        ErrorFont: the font for error messages
        Font: the font for normal messages
        KeepOnPause: option to keep existing message after using the pause method
        PanelHeight: height of the display panel if manually controlling it
        PanelLeft: left side of the panel if manually positioning it
        Margin: pixels to provide spacing for messages when AutoSize is true
        MaxPause: maximum number os seconds to use when DisplayInfoPause is
                  used. This keeps you from typing errors in your code and
                  having a message displayed indefinately. (default 10 seconds)
                  Set this to zero to ignore the maximum seconds.
        MinWidth: minimum width of panel when AutoSize is true (default 300 pixels)
        PanelTop: top of the panel when manually controlling it
        PanelWidth: width of the panel when manually controlling it
    *)
    property AutoCenter: boolean read fAutoCenter write fAutoCenter;
    property AutoSize: boolean read fAutoSize write fAutoSize;
    property Color: TColor read fColor write fColor;
    property DotsInterval: double read fDotsInterval write fDotsInterval;
    property DotsOn: boolean read fDotsOn write fDotsOn;
    property ErrorColor: TColor read eColor write eColor;
    property ErrorFont: TFont read eFont write eFont;
    property Font: TFont read fFont write fFont;
    property KeepOnPause: boolean read fKeepPause write fKeepPause;
    property PanelHeight: integer read fHeight write fHeight;
    property PanelLeft: integer read fLeft write fLeft;
    property Margin: integer read fMargin write fMargin;
    property MaxPause: integer read fMaxPause write fMaxPause;
    property MinWidth: integer read fMinWidth write fMinWidth;
    property PanelTop: integer read fTop write fTop;
    property PanelWidth: integer read fWidth write fWidth;
    property OnChange: TNotifyEvent read fOnChange write fOnChange;
    property OnDestroy: TNotifyEvent read fOnDestroy write fOnDestroy;
  end;

procedure Register;

implementation

var
  AreDotsOn: boolean;

procedure Register;
begin
  RegisterComponents('Additional', [TMessagePanel]);
end;

constructor TMessagePanel.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  fOwner := AOwner;
  fAutoSize := true;
  fAutoCenter := true;
  eFont := TFont.Create;
  fFont := TFont.Create;
  fFont.Name := 'Arial';
  fFont.Size := 14;
  fFont.Style := [fsBold];
  fFont.Color := clYellow;
  eFont.Assign(fFont);
  eFont.Color := clWhite;
  eColor := clRed;
  fColor := clNavy;
  fHeight := 80;
  fWidth := 260;
  fMargin := 10;
  fMaxPause := 10;
  fMinWidth := 300;
  fErrorCode := false;
  fKeepPause := true;
  fDotsOn := false;
  fTimer := nil;
  fDotLabel := nil;
  fDotsInterval := 0.5;
  DisplayPanel := nil;
end;

destructor TMessagePanel.Destroy;
begin
  if Assigned(fOnDestroy) then
    fOnDestroy(self);
  if Assigned(fDotLabel) then
    fDotLabel.Free;
  if Assigned(fTimer) then
    fTimer.Free;
  if Assigned(DisplayPanel) then
    DisplayPanel.Free;
  fFont.Free;
  inherited;
end;

procedure TMessagePanel.DisplayInfo(s: string; Error: boolean);
var
  x: integer;
  GoodOwner: boolean;

begin
  fMessageInfo := s;
  fErrorCode := Error;
  if Assigned(fOnChange) then
    fOnChange(self);
  if (s = '') then begin
    if Assigned(fDotLabel) then
      fDotLabel.Free;
    fDotLabel := nil;
    DisplayPanel.Free;
    DisplayPanel := nil;
    if Assigned(fTimer) then begin
      fTimer.Enabled := false;
      fTimer.OnTimer := nil;
      fTimer.Free;
      fTimer := nil;
    end;
    exit;
  end;
  // make sure the owner is a valid window control
  GoodOwner := Assigned(fOwner) and (fOwner is TControl);
  if not Assigned(DisplayPanel) then begin
    DisplayPanel := TPanel.Create(nil);
    with DisplayPanel do begin
      Top := 0;
      Left := 0;
      Height := 1;
      Width := 1;
      BevelInner := bvLowered;
      BevelWidth := 3;
      if GoodOwner then
        Parent := TWinControl(fOwner);
      Enabled := false;
      BringToFront;
    end;
  end;
  if Error then begin
    DisplayPanel.Color := eColor;
    DisplayPanel.Font.Assign(eFont);
  end else begin
    DisplayPanel.Color := fColor;
    DisplayPanel.Font.Assign(fFont);
  end;
	// set panel caption
	DisplayPanel.Caption := s;
  Application.ProcessMessages;
  if fAutoSize and GoodOwner then begin
    DisplayPanel.Height := MyPnl(DisplayPanel).Canvas.TextHeight(s) * 3;
    if fDotsOn then
      x := MyPnl(DisplayPanel).Canvas.TextWidth(s + ' ...')
    else
      x := MyPnl(DisplayPanel).Canvas.TextWidth(s);
    if x < fMinWidth then
  	  x := fMinWidth;
    if (x + (fMargin * 2)) < TControl(fOwner).Width then
  	  DisplayPanel.Width := x + (fMargin * 2)
    else
  	  DisplayPanel.Width := TControl(fOwner).Width;
  end else begin
    DisplayPanel.Height := fHeight;
    DisplayPanel.Width := fWidth;
  end;
  if fAutoCenter and GoodOwner then begin
    DisplayPanel.Left := ((TControl(fOwner).Width - DisplayPanel.Width) div 2);
    DisplayPanel.Top := ((TControl(fOwner).Height - DisplayPanel.Height) div 2);
  end else begin
    DisplayPanel.Left := fLeft;
    DisplayPanel.Top := fTop;
  end;
  if GoodOwner then
    DisplayPanel.Show;
  Application.ProcessMessages;
  if fDotsOn then begin
    if not Assigned(fDotLabel) then begin
      fDotLabel := TLabel.Create(nil);
      fDotLabel.Parent := DisplayPanel;
      fDotLabel.Caption := '...';
      fDotLabel.Autosize := true;
      fDotLabel.Alignment := taLeftJustify;
      fDotLabel.Visible := false;
    end;
    if not Assigned(fTimer) then begin
      fTimer := TTimer.Create(self);
      fTimer.Interval := Trunc(fDotsInterval * 1000);
      fTimer.OnTimer := DotTimer;
      fTimer.Enabled := true;
    end;
    // place the dots at the end of the message
    fDotLabel.Left := DisplayPanel.Width -
      (((DisplayPanel.Width - MyPnl(DisplayPanel).Canvas.TextWidth(s)) div 2) - 2);
    // center the label
    fDotLabel.Top := (DisplayPanel.Height - fDotLabel.Height) div 2;
  end;
  Application.ProcessMessages;
end;

procedure TMessagePanel.DisplayInfoPause(s: string; m: double; Error: boolean);
var
  oldError: boolean;
  oldMsg: string;
  n: TDateTime;

begin
  oldError := fErrorCode;
  oldMsg := fMessageInfo;
  DisplayInfo(s, Error);
  if (m > fMaxPause) and (fMaxPause > 0) then
    m := fMaxPause;
  m := m * (1 / (60 * 60 * 24));
  n := Now + m;
  while (Now < n) do
    Application.ProcessMessages;
  if fKeepPause then
    DisplayInfo(oldMsg, oldError)
  else
    DisplayInfo('');
end;

function TMessagePanel.GetDisplayInfo: string;
begin
  result := fMessageInfo;
end;

procedure TMessagePanel.DotTimer(Sender: TObject);
begin
 // leave this for timer to display blinking dots...
  if Assigned(fDotLabel) then
    fDotLabel.Visible := AreDotsOn;
  AreDotsOn := not AreDotsOn;
end;
end.
